<?php

namespace FiloBlu\Esb\Setup;

use Magento\Framework\DB\AggregatedFieldDataConverter;
use Magento\Framework\DB\DataConverter\SerializedToJson;
use Magento\Framework\DB\FieldToConvert;
use Magento\Framework\DB\Query\Generator;
use Magento\Framework\DB\Select\QueryModifierFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Exception\AlreadyExistsException;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\Order\StatusFactory;
use Magento\Sales\Model\ResourceModel\Order\Status\Collection as OrderStatusCollection;

/**
 * Class UpgradeData
 * @package FiloBlu\Esb\Setup
 */
class UpgradeData implements UpgradeDataInterface
{
    const STATE_HOLDED = 'holded';
    const STATE_NEW = 'new';
    const STATE_PAYMENT_REVIEW = 'payment_review';
    const STATE_PENDING_PAYMENT = 'pending_payment';
    const STATE_PROCESSING = 'processing';

    /**
     * @var AggregatedFieldDataConverter
     */
    private $aggregatedFieldConverter;

    /**
     * @var QueryModifierFactory
     */
    private $queryModifierFactory;

    /**
     * @var Generator
     */
    private $queryGenerator;

    /**
     * @var StatusFactory
     */
    protected $statusFactory;

    /**
     * @var OrderStatusCollection
     */
    protected $orderStatusCollection;

    /**
     * UpgradeData constructor.
     * @param AggregatedFieldDataConverter $aggregatedFieldConverter
     * @param QueryModifierFactory $queryModifierFactory
     * @param Generator $queryGenerator
     * @param StatusFactory $statusFactory
     * @param OrderStatusCollection $orderStatusCollection
     */
    public function __construct(
        AggregatedFieldDataConverter $aggregatedFieldConverter,
        QueryModifierFactory $queryModifierFactory,
        Generator $queryGenerator,
        StatusFactory $statusFactory,
        OrderStatusCollection $orderStatusCollection
    )
    {
        $this->aggregatedFieldConverter = $aggregatedFieldConverter;
        $this->queryModifierFactory = $queryModifierFactory;
        $this->queryGenerator = $queryGenerator;
        $this->statusFactory = $statusFactory;
        $this->orderStatusCollection = $orderStatusCollection;
    }

    /**
     * {@inheritdoc}
     */
    public function upgrade(
        ModuleDataSetupInterface $setup,
        ModuleContextInterface $context
    )
    {
        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $confirmedStatus = $this->statusFactory->create();
            $confirmedStatus->setData('status', 'confirmed');
            $confirmedStatus->setData('label', 'Confirmed');
            $confirmedStatus->save();
            $confirmedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $orderExportedStatus = $this->statusFactory->create();
            $orderExportedStatus->setData('status', 'order_exported');
            $orderExportedStatus->setData('label', 'Order Exported');
            $orderExportedStatus->save();
            $orderExportedStatus->assignState(Order::STATE_PROCESSING, false, false);

            $paymentConfirmedStatus = $this->statusFactory->create();
            $paymentConfirmedStatus->setData('status', 'payment_confirmed');
            $paymentConfirmedStatus->setData('label', 'Payment Confirmed');
            $paymentConfirmedStatus->save();
            $paymentConfirmedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $paymentExportedStatus = $this->statusFactory->create();
            $paymentExportedStatus->setData('status', 'payment_exported');
            $paymentExportedStatus->setData('label', 'Payment Exported');
            $paymentExportedStatus->save();
            $paymentExportedStatus->assignState(Order::STATE_PROCESSING, false, false);

            $invoicedStatus = $this->statusFactory->create();
            $invoicedStatus->setData('status', 'invoiced');
            $invoicedStatus->setData('label', 'Invoiced');
            $invoicedStatus->save();
            $invoicedStatus->assignState(Order::STATE_PROCESSING, false, true);

            $shippedStatus = $this->statusFactory->create();
            $shippedStatus->setData('status', 'shipped');
            $shippedStatus->setData('label', 'Shipped');
            $shippedStatus->save();
            $shippedStatus->assignState(Order::STATE_PROCESSING, false, true);

        }

        if (version_compare($context->getVersion(), '1.0.21') < 0) {
            $newStatus = $this->statusFactory->create();
            $newStatus->setData('status', 'sent_to_logistics');
            $newStatus->setData('label', 'Sent to Logistics');
            $newStatus->save();
            $newStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);
        }

        if (version_compare($context->getVersion(), '1.1.9', '<')) {
            $this->convertSerializedDataToJson($setup);
        }

        if(version_compare($context->getVersion(), '1.1.11') < 0) {
            $newStatus = $this->statusFactory->create();
            $newStatus->setData('status', 'blocked');
            $newStatus->setData('label', 'Blocked');
            try{
                $newStatus->save();
            } catch (AlreadyExistsException $e) {}

            try{
                $newStatus->assignState(self::STATE_HOLDED, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_NEW, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_PAYMENT_REVIEW, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_PENDING_PAYMENT, false, true);
            }catch (AlreadyExistsException $e){}

            try{
                $newStatus->assignState(self::STATE_PROCESSING, false, true);
            }catch (AlreadyExistsException $e){}
        }

        if(version_compare($context->getVersion(), '2.0.0') < 0) {
            $this->addNewEsbOrderStatuses();
        }

    }

    private function addNewEsbOrderStatuses(){

        $this->createOrderStatusIfNotExists(
            'waiting_confirmation_by_ext',
            'Waiting Confirmation By Ext',
            Order::STATE_PROCESSING
        );
        $this->createOrderStatusIfNotExists(
            'payment_confirmed_on_ext',
            'Payment Confirmed On Ext',
            Order::STATE_PROCESSING
        );
        $this->createOrderStatusIfNotExists(
            'confirmed_by_ext',
            'Confirmed By Ext',
            Order::STATE_PROCESSING
        );
        $this->createOrderStatusIfNotExists(
            'canceled_by_ext',
            'Canceled By Ext',
            Order::STATE_CANCELED
        );

    }

    /**
     * Upgrade to version 1.1.9, convert the configuration data
     * (oad/general/order_data_fields and oad/general/item_data_fields in core_config_data table).
     *
     * @param ModuleDataSetupInterface $setup
     * @return void
     */
    private function convertSerializedDataToJson(ModuleDataSetupInterface $setup)
    {

        // Convert data for the configuration
        $queryModifier = $this->queryModifierFactory->create(
            'in',
            [
                'values' => [
                    'path' => [
                        'esb/endpoints/events',
                        'esb/ack/flows',
                        'esb/gwmapping/gwproducts'
                    ]
                ]
            ]
        );
        $fieldsToUpdate[] = new FieldToConvert(
            SerializedToJson::class,
            $setup->getTable('core_config_data'),
            'config_id',
            'value',
            $queryModifier
        );

        $this->aggregatedFieldConverter->convert($fieldsToUpdate, $setup->getConnection());
    }

    public function createOrderStatusIfNotExists($status,$label,$state){
        $orderStatusesArray = $this->orderStatusCollection->getAllIds();

        if(in_array($status,$orderStatusesArray)){
            return false;
        }

        $newStatus = $this->statusFactory->create();
        $newStatus->setData('status', $status);
        $newStatus->setData('label', $label);
        $newStatus->save();
        $newStatus->assignState($state, false, true);

    }

}